﻿Namespace SudukoArticle.Utility
    ''' <summary>
    ''' Ported from http://norvig.com/sudo.py by Richard Birkby, June 2007.
    ''' See http://norvig.com/sudoku.html
    ''' Also, https://bugzilla.mozilla.org/attachment.cgi?id=266577 - Javascript1.8 version
    ''' </summary>
    Friend Module LinqSudokuSolver
        ' Throughout this program we have:
        '   r is a row,    e.g. 'A'
        '   c is a column, e.g. '3'
        '   s is a square, e.g. 'A3'
        '   d is a digit,  e.g. '9'
        '   u is a unit,   e.g. ['A1','B1','C1','D1','E1','F1','G1','H1','I1']
        '   g is a grid,   e.g. 81 non-blank chars, e.g. starting with '.18...7...
        '   values is a dict of possible values, e.g. {'A1':'123489', 'A2':'8', ...}
        Private rows As String = "ABCDEFGHI"
        Private cols As String = "123456789"
        Private digits As String = "123456789"
        Private squares() As String = cross(rows, cols)
        Private peers As Dictionary(Of String, IEnumerable(Of String))
        Private units As Dictionary(Of String, IEnumerable(Of String))

        '        
        '         * def cross(A, B):
        '         *   return [a+b for a in A for b in B]
        '         
        Private Function cross(ByVal A As String, ByVal B As String) As String()
            Return (
                From _a In A, _b In B
                Select "" & A + B).ToArray()
        End Function

        Sub New()
            'unitlist = ((cross(rows, c) for c in cols) +
            '    (cross(r, cols) for r in rows) + 
            '    (cross(rs, cs) for rs in ( 'ABC","cDEF","cGHI') for cs in ('123","c456","c789')])

            Dim unitlist = ((
                From c In cols
                Select cross(rows, c.ToString())).Concat(
                From r In rows
                Select cross(r.ToString(), cols)).Concat(
                From rs In ({"ABC", "DEF", "GHI"}), cs In ({"123", "456", "789"})
                Select cross(rs, cs)))

            'units = dict((s, (u for u in unitlist if s in u))
            '    for s in squares)


            Dim units = (From s In squares, u In unitlist
                  Where u.Contains(s)
                  Group u By s Into g = Group
                  Select New With {.key = s, .value = g}).ToDictionary(Function(c) c.key, Function(c) c.value)

            'peers = dict((s, set(s2 for u in units(s) 
            '  for s2 in u if s2 <> s)) for s in squares)

            peers = (From s In squares, u In units(s), s2 In u
                  Where s2 <> s
                  Group s2 By s Into g = Group
                  Order By g.Distinct()
                  Select New With {.key = s, .value = g.Distinct()}
                  ).ToDictionary(Function(c) c.key, Function(c) c.value)

        End Sub
        '       [Javascript1.8]
        '       * function zip(A, B) {
        '       *   let z = []
        '       *   let n = Math.min(A.length, B.length)
        '       *   for (let i = 0; i < n; i++)
        '       *     z.push([A[i], B[i]])
        '       *   return z
        '       * }
        ''         
        Private Function zip(ByVal A() As String, ByVal B() As String) As String()()
            Dim n = Math.Min(A.Length, B.Length)
            Dim sd(n - 1)() As String
            For i = 0 To n - 1
                sd(i) = New String() {A(i).ToString(), B(i).ToString()}
            Next i
            Return sd
        End Function
        '        
        '        def parse_grid(grid):
        '            "Given a string of 81 digits (or . or 0 or -), return a dict of {cell:values}"
        '            grid = [c for c in grid if c in '0.-123456789']
        '            values = dict((s, digits) for s in squares) ## To start, every square can be any digit
        '            for s,d in zip(squares, grid):
        '                if d in digits and not assign(values, s, d):
        '                return False
        '            return values
        '        
        ''' <summary>Given a string of 81 digits (or . or 0 or -), return a dict of {cell:values}</summary>
        Public Function parse_grid(ByVal grid As String) As Dictionary(Of String, String)
            Dim grid2 = From c In grid
                        Where "0.-123456789".Contains(c)
                        Select c
            Dim values = squares.ToDictionary(Function(s) s, Function(s) digits) 'To start, every square can be any digit

            For Each sd In zip(squares, (From s In grid Select CStr(s)).ToArray())

                Dim s = sd(0)
                Dim d = sd(1)

                If digits.Contains(d) AndAlso assign(values, s, d) Is Nothing Then
                    Return Nothing
                End If
            Next sd
            Return values
        End Function

        '        
        '         * def search(values):
        '         *   "Using depth-first search and propagation, try all possible values."
        '         *   if values is False:
        '         *     return False ## Failed earlier
        '         *   if all(len(values[s]) == 1 for s in squares): 
        '         *     return values ## Solved!
        '         *   ## Chose the unfilled square s with the fewest possibilities
        '         *   _,s = min((len(values[s]), s) for s in squares if len(values[s]) > 1)
        '         *   return some(search(assign(values.copy(), s, d)) 
        '         *           for d in values[s])
        '         
        ''' <summary>Using depth-first search and propagation, try all possible values.</summary>
        Public Function search(ByVal values As Dictionary(Of String, String)) As Dictionary(Of String, String)
            If values Is Nothing Then
                Return Nothing ' Failed earlier
            End If
            If all(
                From s In squares
                Select If(values(s).Length = 1, "", Nothing)) Then
                Return values ' Solved!
            End If

            ' Chose the unfilled square s with the fewest possibilities
            Dim s2 = (
                From s In squares
                Where values(s).Length > 1
                Order By values(s).Length Ascending
                Select s).First()

            Return some(
                From d In values(s2)
                Select search(assign(New Dictionary(Of String, String)(values), s2, d.ToString())))
        End Function

        '        
        '         * def assign(values, s, d):
        '         *   "Eliminate all the other values (except d) from values[s] and propagate."
        '         *   if all(eliminate(values, s, d2) for d2 in values[s] if d2 != d):
        '         *     return values
        '         *   else:
        '         *     return False
        '         
        ''' <summary>Eliminate all the other values (except d) from values[s] and propagate.</summary>
        Private Function assign(ByVal values As Dictionary(Of String, String),
                                ByVal s As String,
                                ByVal d As String) As Dictionary(Of String, String)
            If all(
                From d2 In values(s)
                Where d2.ToString() <> d
                Select eliminate(values, s, d2.ToString())) Then
                Return values
            End If
            Return Nothing
        End Function

        ' Eliminate d from values[s]; propagate when values or places <= 2.
        '         def eliminate(values, s, d):
        '         *   "Eliminate d from values[s]; propagate when values or places <= 2."
        '         *   if d not in values[s]:
        '         *       return values ## Already eliminated
        '         *   values[s] = values[s].replace(d,'')
        '         *   if len(values[s]) == 0:
        '         *       return False ## Contradiction: removed last value
        '         *   elif len(values[s]) == 1:
        '         *       ## If there is only one value (d2) left in square, remove it from peers
        '         *       d2, = values[s]
        '         *       if not all(eliminate(values, s2, d2) for s2 in peers[s]):
        '         *           return False
        '         *   ## Now check the places where d appears in the units of s
        '         *   for u in units[s]:
        '         *       dplaces = [s for s in u if d in values[s]]
        '         *       if len(dplaces) == 0:
        '         *           return False
        '         *       elif len(dplaces) == 1:
        '         *           # d can only be in one place in unit; assign it there
        '         *           if not assign(values, dplaces[0], d):
        '         *               return False
        '         *   return values
        '         
        ''' <summary>Eliminate d from values[s]; propagate when values or places &lt;= 2.</summary>
        Private Function eliminate(ByVal values As Dictionary(Of String, String),
                                   ByVal s As String, ByVal d As String) As Dictionary(Of String, String)
            If Not values(s).Contains(d) Then
                Return values
            End If
            values(s) = values(s).Replace(d, "")
            If values(s).Length = 0 Then
                Return Nothing 'Contradiction: removed last value
            ElseIf values(s).Length = 1 Then
                'If there is only one value (d2) left in square, remove it from peers
                Dim d2 = values(s)
                If Not all(
                    From s2 In peers(s)
                    Select eliminate(values, s2, d2)) Then
                    Return Nothing
                End If
            End If

            'Now check the places where d appears in the units of s
            For Each u In units(s)
                Dim dplaces = From s2 In u
                              Where values(s2).Contains(d)
                              Select s2
                If dplaces.Count() = 0 Then
                    Return Nothing
                ElseIf dplaces.Count() = 1 Then
                    ' d can only be in one place in unit; assign it there
                    If assign(values, dplaces.First(), d) Is Nothing Then
                        Return Nothing
                    End If
                End If
            Next u
            Return values
        End Function

        '        
        '         * def all(seq):
        '         *   for e in seq:
        '         *     if not e: return False
        '         *   return True
        '         
        Private Function all(Of T)(ByVal seq As IEnumerable(Of T)) As Boolean
            For Each e In seq
                If e Is Nothing Then
                    Return False
                End If
            Next e
            Return True
        End Function

        '        
        '         * def some(seq):
        '         *   for e in seq:
        '         *     if e: return e
        '         *  return False
        '         
        Private Function some(Of T)(ByVal seq As IEnumerable(Of T)) As T
            For Each e In seq
                If e IsNot Nothing Then
                    Return e
                End If
            Next e
            Return Nothing
        End Function
        '        
        '         * def center(s, width):
        '         *   n = width - len(s)
        '         *   if n <= 0: return s
        '         *   half = n/2
        '         *   if n%2 and width%2:
        '         *     half = half+1
        '         *   return ' '*half +  s + ' '*(n-half)
        '         
        <System.Runtime.CompilerServices.Extension()>
        Private Function Center(ByVal s As String, ByVal width As Integer) As String
            Dim n = width - s.Length
            If n <= 0 Then
                Return s
            End If
            Dim half = n \ 2

            If n Mod 2 > 0 AndAlso width Mod 2 > 0 Then
                half += 1
            End If

            Return New String(" "c, half) & s & New String(" "c, n - half)
        End Function
        '        
        '         * def printboard(values):
        '         *   "Used for debugging."
        '         *   width = 1+max(len(values[s]) for s in squares)
        '         *   line = '\n' + '+'.join(['-'*(width*3)]*3)
        '         *   for r in rows:
        '         *     print ''.join(values[r+c].center(width)+(c in '36' and '|' or '')
        '         *            for c in cols) + (r in 'CF' and line or '')
        '         *   print
        '         *   return values
        '         
        ''' <summary>Used for debugging.</summary>
        Public Function print_board(ByVal values As Dictionary(Of String, String)) As Dictionary(Of String, String)
            If values Is Nothing Then
                Return Nothing
            End If

            Dim width = 1 + (
                From s In squares
                Select values(s).Length).Max()
            Dim line = vbLf & String.Join("+", Enumerable.Repeat(New String("-"c, width * 3), 3).ToArray())

            For Each r In rows
                Dim Myr = r
                Debug.WriteLine(String.Join("", (
                    From c In cols
                    Select values("" & Myr & c).Center(width) + (If("36".Contains(c), "|", ""))).ToArray()) + (If("CF".Contains(r), line, "")))
            Next r

            Console.WriteLine()
            Return values
        End Function

        Public Sub Test()
            Dim easy = "..3.2.6..9..3.5..1..18.64....81.29..7.......8..67.82....26.95..8..2.3..9..5.1.3.."
            print_board(parse_grid(easy))

            Console.WriteLine("Simple elimination not possible:")
            Dim grid = "4.....8.5.3..........7......2.....6.....8.4......1.......6.3.7.5..2.....1.4......"
            print_board(parse_grid(grid))

            Console.WriteLine("Try again with search:")
            print_board(search(parse_grid(grid)))

            Dim hardest = "85...24..72......9..4.........1.7..23.5...9...4...........8..7..17..........36.4."

            Dim start As Date = Date.Now
            For i = 0 To 299
                search(parse_grid(hardest))
            Next i
            Console.WriteLine("Solving 'hardest' sodoku took on average " &
                              (Date.Now.Subtract(start)).TotalMilliseconds / 300 &
                              " milliseconds, timed over 300 runs")


            Dim top94 =
                "4.....8.5.3..........7......2.....6.....8.4......1.......6.3.7.5..2.....1.4......" & vbCrLf &
                "52...6.........7.13...........4..8..6......5...........418.........3..2...87....." & vbCrLf &
                "6.....8.3.4.7.................5.4.7.3..2.....1.6.......2.....5.....8.6......1...." & vbCrLf &
                "48.3............71.2.......7.5....6....2..8.............1.76...3.....4......5...." & vbCrLf &
                "....14....3....2...7..........9...3.6.1.............8.2.....1.4....5.6.....7.8..." & vbCrLf &
                "......52..8.4......3...9...5.1...6..2..7........3.....6...1..........7.4.......3." & vbCrLf &
                "6.2.5.........3.4..........43...8....1....2........7..5..27...........81...6....." & vbCrLf &
                ".524.........7.1..............8.2...3.....6...9.5.....1.6.3...........897........" & vbCrLf &
                "6.2.5.........4.3..........43...8....1....2........7..5..27...........81...6....." & vbCrLf &
                ".923.........8.1...........1.7.4...........658.........6.5.2...4.....7.....9....." & vbCrLf &
                "6..3.2....5.....1..........7.26............543.........8.15........4.2........7.." & vbCrLf &
                ".6.5.1.9.1...9..539....7....4.8...7.......5.8.817.5.3.....5.2............76..8..." & vbCrLf &
                "..5...987.4..5...1..7......2...48....9.1.....6..2.....3..6..2.......9.7.......5.." & vbCrLf &
                "3.6.7...........518.........1.4.5...7.....6.....2......2.....4.....8.3.....5....." & vbCrLf &
                "1.....3.8.7.4..............2.3.1...........958.........5.6...7.....8.2...4......." & vbCrLf &
                "6..3.2....4.....1..........7.26............543.........8.15........4.2........7.." & vbCrLf &
                "....3..9....2....1.5.9..............1.2.8.4.6.8.5...2..75......4.1..6..3.....4.6." & vbCrLf &
                "45.....3....8.1....9...........5..9.2..7.....8.........1..4..........7.2...6..8.." & vbCrLf &
                ".237....68...6.59.9.....7......4.97.3.7.96..2.........5..47.........2....8......." & vbCrLf &
                "..84...3....3.....9....157479...8........7..514.....2...9.6...2.5....4......9..56" & vbCrLf &
                ".98.1....2......6.............3.2.5..84.........6.........4.8.93..5...........1.." & vbCrLf &
                "..247..58..............1.4.....2...9528.9.4....9...1.........3.3....75..685..2..." & vbCrLf &
                "4.....8.5.3..........7......2.....6.....5.4......1.......6.3.7.5..2.....1.9......" & vbCrLf &
                ".2.3......63.....58.......15....9.3....7........1....8.879..26......6.7...6..7..4" & vbCrLf &
                "1.....7.9.4...72..8.........7..1..6.3.......5.6..4..2.........8..53...7.7.2....46" & vbCrLf &
                "4.....3.....8.2......7........1...8734.......6........5...6........1.4...82......" & vbCrLf &
                ".......71.2.8........4.3...7...6..5....2..3..9........6...7.....8....4......5...." & vbCrLf &
                "6..3.2....4.....8..........7.26............543.........8.15........8.2........7.." & vbCrLf &
                ".47.8...1............6..7..6....357......5....1..6....28..4.....9.1...4.....2.69." & vbCrLf &
                "......8.17..2........5.6......7...5..1....3...8.......5......2..4..8....6...3...." & vbCrLf &
                "38.6.......9.......2..3.51......5....3..1..6....4......17.5..8.......9.......7.32" & vbCrLf &
                "...5...........5.697.....2...48.2...25.1...3..8..3.........4.7..13.5..9..2...31.." & vbCrLf &
                ".2.......3.5.62..9.68...3...5..........64.8.2..47..9....3.....1.....6...17.43...." & vbCrLf &
                ".8..4....3......1........2...5...4.69..1..8..2...........3.9....6....5.....2....." & vbCrLf &
                "..8.9.1...6.5...2......6....3.1.7.5.........9..4...3...5....2...7...3.8.2..7....4" & vbCrLf &
                "4.....5.8.3..........7......2.....6.....5.8......1.......6.3.7.5..2.....1.8......" & vbCrLf &
                "1.....3.8.6.4..............2.3.1...........958.........5.6...7.....8.2...4......." & vbCrLf &
                "1....6.8..64..........4...7....9.6...7.4..5..5...7.1...5....32.3....8...4........" & vbCrLf &
                "249.6...3.3....2..8.......5.....6......2......1..4.82..9.5..7....4.....1.7...3..." & vbCrLf &
                "...8....9.873...4.6..7.......85..97...........43..75.......3....3...145.4....2..1" & vbCrLf &
                "...5.1....9....8...6.......4.1..........7..9........3.8.....1.5...2..4.....36...." & vbCrLf &
                "......8.16..2........7.5......6...2..1....3...8.......2......7..3..8....5...4...." & vbCrLf &
                ".476...5.8.3.....2.....9......8.5..6...1.....6.24......78...51...6....4..9...4..7" & vbCrLf &
                ".....7.95.....1...86..2.....2..73..85......6...3..49..3.5...41724................" & vbCrLf &
                ".4.5.....8...9..3..76.2.....146..........9..7.....36....1..4.5..6......3..71..2.." & vbCrLf &
                ".834.........7..5...........4.1.8..........27...3.....2.6.5....5.....8........1.." & vbCrLf &
                "..9.....3.....9...7.....5.6..65..4.....3......28......3..75.6..6...........12.3.8" & vbCrLf &
                ".26.39......6....19.....7.......4..9.5....2....85.....3..2..9..4....762.........4" & vbCrLf &
                "2.3.8....8..7...........1...6.5.7...4......3....1............82.5....6...1......." & vbCrLf &
                "6..3.2....1.....5..........7.26............843.........8.15........8.2........7.." & vbCrLf &
                "1.....9...64..1.7..7..4.......3.....3.89..5....7....2.....6.7.9.....4.1....129.3." & vbCrLf &
                ".........9......84.623...5....6...453...1...6...9...7....1.....4.5..2....3.8....9" & vbCrLf &
                ".2....5938..5..46.94..6...8..2.3.....6..8.73.7..2.........4.38..7....6..........5" & vbCrLf &
                "9.4..5...25.6..1..31......8.7...9...4..26......147....7.......2...3..8.6.4.....9." & vbCrLf &
                "...52.....9...3..4......7...1.....4..8..453..6...1...87.2........8....32.4..8..1." & vbCrLf &
                "53..2.9...24.3..5...9..........1.827...7.........981.............64....91.2.5.43." & vbCrLf &
                "1....786...7..8.1.8..2....9........24...1......9..5...6.8..........5.9.......93.4" & vbCrLf &
                "....5...11......7..6.....8......4.....9.1.3.....596.2..8..62..7..7......3.5.7.2.." & vbCrLf &
                ".47.2....8....1....3....9.2.....5...6..81..5.....4.....7....3.4...9...1.4..27.8.." & vbCrLf &
                "......94.....9...53....5.7..8.4..1..463...........7.8.8..7.....7......28.5.26...." & vbCrLf &
                ".2......6....41.....78....1......7....37.....6..412....1..74..5..8.5..7......39.." & vbCrLf &
                "1.....3.8.6.4..............2.3.1...........758.........7.5...6.....8.2...4......." & vbCrLf &
                "2....1.9..1..3.7..9..8...2.......85..6.4.........7...3.2.3...6....5.....1.9...2.5" & vbCrLf &
                "..7..8.....6.2.3...3......9.1..5..6.....1.....7.9....2........4.83..4...26....51." & vbCrLf &
                "...36....85.......9.4..8........68.........17..9..45...1.5...6.4....9..2.....3..." & vbCrLf &
                "34.6.......7.......2..8.57......5....7..1..2....4......36.2..1.......9.......7.82" & vbCrLf &
                "......4.18..2........6.7......8...6..4....3...1.......6......2..5..1....7...3...." & vbCrLf &
                ".4..5..67...1...4....2.....1..8..3........2...6...........4..5.3.....8..2........" & vbCrLf &
                ".......4...2..4..1.7..5..9...3..7....4..6....6..1..8...2....1..85.9...6.....8...3" & vbCrLf &
                "8..7....4.5....6............3.97...8....43..5....2.9....6......2...6...7.71..83.2" & vbCrLf &
                ".8...4.5....7..3............1..85...6.....2......4....3.26............417........" & vbCrLf &
                "....7..8...6...5...2...3.61.1...7..2..8..534.2..9.......2......58...6.3.4...1...." & vbCrLf &
                "......8.16..2........7.5......6...2..1....3...8.......2......7..4..8....5...3...." & vbCrLf &
                ".2..........6....3.74.8.........3..2.8..4..1.6..5.........1.78.5....9..........4." & vbCrLf &
                ".52..68.......7.2.......6....48..9..2..41......1.....8..61..38.....9...63..6..1.9" & vbCrLf &
                "....1.78.5....9..........4..2..........6....3.74.8.........3..2.8..4..1.6..5....." & vbCrLf &
                "1.......3.6.3..7...7...5..121.7...9...7........8.1..2....8.64....9.2..6....4....." & vbCrLf &
                "4...7.1....19.46.5.....1......7....2..2.3....847..6....14...8.6.2....3..6...9...." & vbCrLf &
                "......8.17..2........5.6......7...5..1....3...8.......5......2..3..8....6...4...." & vbCrLf &
                "963......1....8......2.5....4.8......1....7......3..257......3...9.2.4.7......9.." & vbCrLf &
                "15.3......7..4.2....4.72.....8.........9..1.8.1..8.79......38...........6....7423" & vbCrLf &
                "..........5724...98....947...9..3...5..9..12...3.1.9...6....25....56.....7......6" & vbCrLf &
                "....75....1..2.....4...3...5.....3.2...8...1.......6.....1..48.2........7........" & vbCrLf &
                "6.....7.3.4.8.................5.4.8.7..2.....1.3.......2.....5.....7.9......1...." & vbCrLf &
                "....6...4..6.3....1..4..5.77.....8.5...8.....6.8....9...2.9....4....32....97..1.." & vbCrLf &
                ".32.....58..3.....9.428...1...4...39...6...5.....1.....2...67.8.....4....95....6." & vbCrLf &
                "...5.3.......6.7..5.8....1636..2.......4.1.......3...567....2.8..4.7.......2..5.." & vbCrLf &
                ".5.3.7.4.1.........3.......5.8.3.61....8..5.9.6..1........4...6...6927....2...9.." & vbCrLf &
                "..5..8..18......9.......78....4.....64....9......53..2.6.........138..5....9.714." & vbCrLf &
                "..........72.6.1....51...82.8...13..4.........37.9..1.....238..5.4..9.........79." & vbCrLf &
                "...658.....4......12............96.7...3..5....2.8...3..19..8..3.6.....4....473.." & vbCrLf &
                ".2.3.......6..8.9.83.5........2...8.7.9..5........6..4.......1...1...4.22..7..8.9" & vbCrLf &
                ".5..9....1.....6.....3.8.....8.4...9514.......3....2..........4.8...6..77..15..6." & vbCrLf &
                ".....2.......7...17..3...9.8..7......2.89.6...13..6....9..5.824.....891.........." & vbCrLf &
                "3...8.......7....51..............36...2..4....7...........6.13..452...........8.."
            Dim top95 = top94.Split(CChar(vbCrLf))
            For Each game In top95
                Console.WriteLine(game)
                print_board(search(parse_grid(game)))
                search(parse_grid(game))
            Next game
            Console.WriteLine("Press enter to finish")
            Console.ReadLine()
        End Sub
    End Module
End Namespace

