﻿Imports System.IO
Imports System.IO.IsolatedStorage
Imports System.Text
Imports System.Xml.Serialization
Imports SudukoArticle.Utility

Namespace SudukoArticle.ViewModels
	Public Class GameBoardViewModel
		Inherits ViewModelBase
        Private Const FileName = "gameboard.dat"

		Public Property SelectedBox() As SquareViewModel
		Public Property GameArray() As SquareViewModel(,)

		Public Sub SendInput(ByVal inputValue As Integer)
            If SelectedBox IsNot Nothing Then
                SelectedBox.Value = inputValue
                ValidateBoard()
                SelectedBox.IsSelected = False
                SelectedBox = Nothing
            End If
		End Sub

		Public Sub SaveToDisk()
            Using store = IsolatedStorageFile.GetUserStoreForApplication()
                If store.FileExists(FileName) Then
                    store.DeleteFile(FileName)
                End If

                Using stream = store.CreateFile(FileName)
                    Using writer As New StreamWriter(stream)
                        Dim s As New List(Of SquareViewModel)()
                        For Each item In GameArray
                            s.Add(item)
                        Next item

                        Dim serializer As New XmlSerializer(s.GetType())
                        serializer.Serialize(writer, s)
                    End Using
                End Using
            End Using
		End Sub

		Public Shared Function LoadFromDisk() As GameBoardViewModel
			Dim result As GameBoardViewModel = Nothing

            Using store = IsolatedStorageFile.GetUserStoreForApplication()
                If store.FileExists(FileName) Then
                    Using stream = store.OpenFile(FileName, FileMode.Open)
                        Using reader As New StreamReader(stream)
                            Dim s As New List(Of SquareViewModel)()
                            Dim serializer As New XmlSerializer(s.GetType())
                            s = CType(serializer.Deserialize(New StringReader(reader.ReadToEnd())), 
                                List(Of SquareViewModel))

                            result = New GameBoardViewModel()
                            result.GameArray = LoadFromSquareList(s)
                        End Using
                    End Using
                End If
            End Using

			Return result
		End Function

		Public Shared Function LoadNewPuzzle() As GameBoardViewModel
			Dim result As New GameBoardViewModel()

			Dim random As New Random()
            Dim easyPuzzle = SavedBoards.EasyGames(random.Next(0, SavedBoards.EasyGames.Length - 1))
			Dim squares As New List(Of SquareViewModel)()
            For Each s In easyPuzzle.ToCharArray()
                Dim square As New SquareViewModel()
                If s <> "."c Then
                    square.Value = Integer.Parse(s.ToString())
                    square.IsEditable = False
                End If
                squares.Add(square)
            Next s

			result.GameArray = LoadFromSquareList(squares)
			Return result
		End Function

		Public Overrides Function ToString() As String
			Dim result As New StringBuilder()
            For Each item In GameArray
                result.Append(item.Value.ToString())
            Next item
			Return result.ToString()
		End Function

		Public Sub Solve()
            SendInput(0) 'deselect item
			Clear()
            Dim x = 0
            Dim y = 0
			Dim solved As Dictionary(Of String, String) = LinqSudokuSolver.parse_grid(Me.ToString())
			LinqSudokuSolver.print_board(solved)
            For Each item In solved
                Dim key As String = item.Key
                Dim value As String = item.Value
                GameArray(y, x).Value = Integer.Parse(item.Value)

                x += 1
                If x = 9 Then
                    x = 0
                    y += 1
                End If
            Next item
		End Sub

		Public Sub Clear()
            For Each item In GameArray
                If item.IsEditable Then
                    item.Value = 0
                    item.IsValid = True
                End If
            Next item
		End Sub

		Private Sub ValidateBoard()
			SelectedBox.IsValid = True
			If SelectedBox.Value > 0 Then
				ValidateRow(SelectedBox.Row)
				ValidateColumn(SelectedBox.Column)
				ValidateGroup(SelectedBox.Row, SelectedBox.Column)
			End If
		End Sub

		Private Sub ValidateRow(ByVal rowNumber As Integer)
            For i = 0 To 8
                If GameArray(rowNumber, i) IsNot SelectedBox Then
                    If SelectedBox.Value = (GameArray(rowNumber, i).Value) Then
                        SelectedBox.IsValid = False
                    End If
                End If
            Next i
		End Sub

		Private Sub ValidateColumn(ByVal columnNumber As Integer)
            For i = 0 To 8
                If GameArray(i, columnNumber) IsNot SelectedBox Then
                    If SelectedBox.Value = (GameArray(i, columnNumber).Value) Then
                        SelectedBox.IsValid = False
                    End If
                End If
            Next i
		End Sub

		''' <summary>
		''' assumes row and column have already been validated, so it doesn't check those squares
		''' </summary>
		Private Sub ValidateGroup(ByVal rowNumber As Integer, ByVal columnNumber As Integer)
            Dim row1 = 0
            Dim row2 = 0
            Dim column1 = 0
            Dim column2 = 0

			Select Case rowNumber Mod 3
				Case 0
					row1 = rowNumber + 1
					row2 = rowNumber + 2
				Case 1
					row1 = rowNumber - 1
					row2 = rowNumber + 1
				Case 2
					row1 = rowNumber - 2
					row2 = rowNumber - 1
			End Select
			Select Case columnNumber Mod 3
				Case 0
					column1 = columnNumber + 1
					column2 = columnNumber + 2
				Case 1
					column1 = columnNumber - 1
					column2 = columnNumber + 1
				Case 2
					column1 = columnNumber - 2
					column2 = columnNumber - 1
			End Select

			If GameArray(row1, column1).Value = SelectedBox.Value Then
				SelectedBox.IsValid = False
			ElseIf GameArray(row1, column2).Value = SelectedBox.Value Then
				SelectedBox.IsValid = False
			ElseIf GameArray(row2, column1).Value = SelectedBox.Value Then
				SelectedBox.IsValid = False
			ElseIf GameArray(row2, column2).Value = SelectedBox.Value Then
				SelectedBox.IsValid = False
			End If
		End Sub

		Private Shared Function LoadFromSquareList(ByVal list As List(Of SquareViewModel)) As SquareViewModel(,)
			Dim result(8, 8) As SquareViewModel
            Dim counter = 0
            For i = 0 To 8
                For j = 0 To 8
                    result(i, j) = list(counter)
                    result(i, j).Row = i
                    result(i, j).Column = j
                    counter += 1
                Next j
            Next i

			Return result
		End Function
	End Class
End Namespace
